package com.ruoyi.web.controller.system;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.PoCollection;
import com.ruoyi.system.service.IPoCollectionService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;

/**
 * 藏品 信息操作处理
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/system/collections")
@Log4j2
public class PoCollectionController extends BaseController {
    @Autowired
    private IPoCollectionService collectionService;

    /**
     * 获取藏品列表
     */
    @PreAuthorize("@ss.hasPermi('system:collection:list')")
    @GetMapping("/list")
    @ApiOperation("藏品列表 查询")
    @Log(title = "藏品", businessType = BusinessType.EXPORT)
    public TableDataInfo list(@RequestBody(required = false) PoCollection poCollection) {
        startPage();
        List<PoCollection> list = collectionService.selectPoCollectionList(poCollection);
        return getDataTable(list);
    }

    /**
     * 获取藏品列表
     */
    @PreAuthorize("@ss.hasPermi('system:collection:list')")
    @GetMapping("/query")
    @ApiOperation("藏品列表 搜索")
    @Log(title = "藏品", businessType = BusinessType.EXPORT)
    public TableDataInfo query(@RequestParam(value = "collectionName", required = false) String collectionName,
                               @RequestParam(value = "startTime", required = false) Date startTime,
                               @RequestParam(value = "endTime", required = false) Date endTime,
                               @RequestParam(value = "status", required = false) Long status,
                               @RequestParam(value = "delFlag", required = false) Long delFlag) {

        startPage();
        List<PoCollection> list = collectionService.queryPoCollectionList(collectionName, startTime, endTime, status, delFlag);
        return getDataTable(list);
    }

    /**
     * 导出藏品列表
     */
    @PreAuthorize("@ss.hasPermi('system:collection:export')")
    @Log(title = "获取藏品列表", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    @ApiOperation("藏品列表导出")
    public void export(HttpServletResponse response,
                       @RequestParam(value = "collectionName", required = false) String collectionName,
                       @RequestParam(value = "startTime", required = false) Date startTime,
                       @RequestParam(value = "endTime", required = false) Date endTime,
                       @RequestParam(value = "status", required = false) Long status,
                       @RequestParam(value = "delFlag", required = false) Long delFlag) {
        List<PoCollection> list = collectionService.queryPoCollectionList(collectionName, startTime, endTime, status, delFlag);
        new ExcelUtil<>(PoCollection.class).exportExcel(response, list, "藏品数据");
    }

    /**
     * 根据藏品编号获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:collection:query')")
    @GetMapping(value = "/get")
    @ApiOperation("藏品详情查询")
    @Log(title = "藏品", businessType = BusinessType.EXPORT)
    public AjaxResult getInfo(@RequestParam("id") Long collectionId) {
        return success(collectionService.selectCollectionById(collectionId));
    }

    /**
     * 新增藏品
     */
    @PreAuthorize("@ss.hasPermi('collection:add')")
    @Log(title = "藏品", businessType = BusinessType.INSERT)
    @ApiOperation("添加藏品")
    @PostMapping("/insert")
    public AjaxResult insert(@Validated @RequestBody PoCollection collection) {
        List<PoCollection> collectionList = collectionService.selectPoCollectionList(collection);
        if (collectionList != null && !collectionList.isEmpty()) {
            for (PoCollection col : collectionList) {
                if (col.equals(collection)) {
                    return error("重复的藏品:" + collection + ",已有藏品:" + collectionList);
                }
            }
        }
        collection.setCreateBy(getUsername());
        collection.setUpdateBy(getUsername());
        return toAjax(collectionService.insertCollection(collection));
    }

    /**
     * 批量新增藏品
     */
    @PreAuthorize("@ss.hasPermi('collection:add')")
    @Log(title = "藏品", businessType = BusinessType.INSERT)
    @ApiOperation("添加藏品")
    @PostMapping("/batchInsert")
    public AjaxResult batchInsert(@Validated @RequestBody List<PoCollection> collectionList) {
        for (PoCollection collection : collectionList) {
            List<PoCollection> colList = collectionService.selectPoCollectionList(collection);
            if (colList == null || colList.isEmpty())
                continue;
            for (PoCollection col : colList) {
                if (col.equals(collection)) {
                    return error("重复的藏品:" + collection + ",已有藏品:" + collectionList);
                }
            }
        }
        return toAjax(collectionService.batchInsertCollection(collectionList));
    }

    /**
     * 修改藏品
     */
    @PreAuthorize("@ss.hasPermi('collection:edit')")
    @Log(title = "藏品", businessType = BusinessType.UPDATE)
    @ApiOperation("编辑藏品")
    @PutMapping("/update")
    public AjaxResult update(@Validated @RequestBody PoCollection collection) {
        log.info(collection);
        PoCollection collection1 = collectionService.selectCollectionById(collection.getCollectionId());
        if (collection1 == null)
            return error("要修改的藏品不存在");
        if (collection1.getColCochain() == 0)
            return error("要修改的藏品已上链");
        collection.setUpdateBy(getUsername());
        return toAjax(collectionService.updateCollection(collection));
    }

    /**
     * 删除藏品
     */
    @PreAuthorize("@ss.hasPermi('system:collection:remove')")
    @Log(title = "删除藏品", businessType = BusinessType.DELETE)
    @DeleteMapping("/delete")
    @ApiOperation("根据藏品ID批量删除藏品")
    public AjaxResult remove(@RequestBody Long[] collectionIds) {
        return toAjax(collectionService.deletePoCollectionByCollectionIds(collectionIds));
    }
}
