package com.ruoyi.system.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.system.domain.PoCollection;

import java.util.Date;
import java.util.List;

/**
 * 消息接口
 */
public interface IPoCollectionService extends IService<PoCollection> {
    /**
     * 获取藏品详细内容
     *
     * @param collectionId 藏品ID
     * @return 藏品信息
     */
    PoCollection selectCollectionById(Long collectionId);

    /**
     * 获取藏品列表
     *
     * @param poCollection 藏品信息
     * @return 藏品列表
     */
    List<PoCollection> selectPoCollectionList(PoCollection poCollection);

    /**
     * 搜索,导出藏品列表
     *
     * @param collectionName 藏品名字(模糊查询)
     * @param startTime      创建时间起始
     * @param endTime        创建时间结束
     * @param status         藏品状态
     * @param delFlag        是否已删除
     * @return 藏品列表
     */
    List<PoCollection> queryPoCollectionList(String collectionName, Date startTime, Date endTime, Long status, Long delFlag);

    /**
     * 增加
     *
     * @param collection 藏品信息
     * @return 操作状态
     */
    int insertCollection(PoCollection collection);

    /**
     * 批量增加
     *
     * @param collectionList 藏品信息
     * @return 操作状态
     */
    int batchInsertCollection(List<PoCollection> collectionList);

    /**
     * 修改
     *
     * @param collection 更新的藏品信息
     * @return 操作状态
     */
    int updateCollection(PoCollection collection);

    /**
     * 删除藏品
     *
     * @param collectionIds 藏品ID
     * @return 操作结果
     */
    int deletePoCollectionByCollectionIds(Long[] collectionIds);

    /**
     * 定时更新藏品状态
     */
    void updateCollectionStatus();
}

