package com.ruoyi.system.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.system.domain.PoCollection;
import com.ruoyi.system.domain.vo.TetherVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.PoTetherMapper;
import com.ruoyi.system.domain.PoTether;
import com.ruoyi.system.service.IPoTetherService;
import org.springframework.transaction.annotation.Transactional;

/**
 * 套系Service业务层处理
 *
 * @author ruoyi
 * @date 2023-02-12
 */
@Service
public class PoTetherServiceImpl extends ServiceImpl<PoTetherMapper,PoTether> implements IPoTetherService
{
    @Autowired
    private PoTetherMapper poTetherMapper;


    @Autowired
    private PoCollectionServiceImpl poCollectionService;


    /**
     * 在售
     * @param poTether 套系 在售
     * @return
     */
    @Override
    public List<PoTether> selectPoTetherSellingList(PoTether poTether)
    {
        return poTetherMapper.selectPoTetherSellingList(poTether);
    }

    /**
     * 按照标题和时间搜索
     * @param tetherName
     * @param tetherTimeStart
     * @param tetherTimeEnd
     * @return
     */
    @Override
    public List<PoTether> selectListByTitleAndTime(String tetherName, Date tetherTimeStart, Date tetherTimeEnd) {

        if (tetherName != null) {
            //标题不为空且开始时间和结束时间都不为空
            if (tetherTimeStart != null && tetherTimeEnd != null) {

                if (DateUtils.getNowDate().before(tetherTimeEnd)){
                    tetherTimeEnd = DateUtils.getNowDate();
                }
                if (tetherTimeStart.before(tetherTimeEnd)) {
                    return poTetherMapper.selectPoTetherListByTitleAndTetherTimeStartAndTetherTimeEnd(tetherName, tetherTimeStart, tetherTimeEnd);
                } else{
                    //有一个时间不为空返回提示信息
                    List list = new ArrayList();
                    list.add("请选择正确的时间段");
                    return list;
                }
            } else {//时间都为空
                return poTetherMapper.selectPoTetherListByTitleAndTetherTimeStartAndTetherTimeEnd(tetherName,tetherTimeStart,tetherTimeEnd);
            }
        }
        //标题为空
        else {
            //时间不为空
            if (tetherTimeStart != null && tetherTimeEnd != null) {
                if (tetherTimeStart.before(tetherTimeEnd)){
                    return poTetherMapper.selectPoTetherListByTitleAndTetherTimeStartAndTetherTimeEnd(tetherName,tetherTimeStart,tetherTimeEnd);
                }else{
                    //有一个时间不为空返回提示信息
                    List list = new ArrayList();
                    list.add("请选择正确的时间段");
                    return list;
                }
            }
        }
        //都为空执行分页查询
        PoTether poTether = new PoTether();
        return poTetherMapper.selectPoTetherList(poTether);
    }

    /**
     * 预售
     * @param poTether
     * @return
     */
    @Override
    public List<PoTether> selectPoTetherAdvanceList(PoTether poTether) {
        return poTetherMapper.selectPoTetherAdvanceList(poTether);

    }

    /**
     * 已过期
     * @param poTether
     * @return
     */
    @Override
    public List<PoTether> selectPoTetherSelledList(PoTether poTether) {
        return poTetherMapper.selectPoTetherSelledList(poTether);
    }

    /**
     * 查询套系列表
     * @param poTether
     * @return
     */
    @Override
    public List<PoTether> selectPoTetherList(PoTether poTether) {
        return poTetherMapper.selectPoTetherList(poTether);
    }

    /**
     * 藏品按钮
     * @param
     * @return
     */
    @Override
    public List<PoCollection> selectPoCollectionListById(Long tetherId) {

        return poTetherMapper.selectPoCollectionListById(tetherId);
    }

    /**
     * 新增套系 同时保存对应的藏品数据到中间表
     *
     * @param tetherVo
     * @return
     */
    @Override
    @Transactional
    public void insertPoTetherWithCollection(TetherVo tetherVo) {
        if (tetherVo.getPoTetherandcollections() == null){
            //保存套系的基本数据到套系表tether
            this.poTetherMapper.insertPoTether(tetherVo);
        }else {

            this.poTetherMapper.insertPoTether(tetherVo);
            //套系包含藏品
            //保存藏品数据到中间表 批量增加
            System.out.println(tetherVo.getPoTetherandcollections().size());


            for (int i = 0; i < tetherVo.getPoTetherandcollections().size(); i++) {
                PoCollection poCollection = new PoCollection();
                poCollection.setCollectionTitle(tetherVo.getPoTetherandcollections().get(i).getCollectionTitle());
                poCollection.setCollectionType(tetherVo.getPoTetherandcollections().get(i).getCollectionType());
                poCollection.setTetherId(tetherVo.getTetherId());
                poCollectionService.insertCollection(poCollection);
            }
        }
    }
    /**
     * 通过id查询套系信息和对应的藏品信息
     * @param tetherId
     * @return
     */
    @Override
    public TetherVo selectPoTetherByTetherIdWithCollection(Long tetherId) {
        //查询套系基本信息 po_tether
        PoTether poTether = this.poTetherMapper.selectPoTetherByTetherId(tetherId);
        TetherVo tetherVo = new TetherVo();
        if (poTether != null) {
            BeanUtils.copyProperties(poTether, tetherVo);
        }
        return  tetherVo;
    }

    /**
     * 修改套系信息
     * @param tetherVo
     * @return
     */
    @Override
    @Transactional
    public void updatePoTetherWithCollection(TetherVo tetherVo) {
        if (tetherVo.getPoTetherandcollections() == null){
            //更新po_tether表基本信息
            this.poTetherMapper.updatePoTether(tetherVo);
        }
       else {

            this.poTetherMapper.updatePoTether(tetherVo);
            //清理po_collection表数据
            LambdaQueryWrapper<PoCollection> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(PoCollection::getTetherId, tetherVo.getTetherId());
            poCollectionService.remove(lambdaQueryWrapper);
//        //添加当前提交过来的藏品数据--po_Tetherandcollection表的insert操作

            for (int i = 0; i < tetherVo.getPoTetherandcollections().size(); i++) {
                System.out.println(tetherVo.getPoTetherandcollections().get(i).getCollectionTitle());
                PoCollection poTetherandcollection1 = new PoCollection();
                poTetherandcollection1.setCollectionTitle(tetherVo.getPoTetherandcollections().get(i).getCollectionTitle());
                poTetherandcollection1.setCollectionType(tetherVo.getPoTetherandcollections().get(i).getCollectionType());
                poTetherandcollection1.setTetherId(tetherVo.getTetherId());
                poCollectionService.insertCollection(poTetherandcollection1);
            }
        }
    }


    /**
     * 同时删除套系和藏品的关联关系
     * @param tetherIds
     * @return
     */
    @Override
    public void deletePoTetherByTetherIdsWithCollection(Long[] tetherIds) throws Exception {
        //查询套系状态确认是否可以删除
        LambdaQueryWrapper<PoTether> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        LambdaQueryWrapper<PoCollection> lambdaQueryWrapper1 = new LambdaQueryWrapper<>();

        //判断套系是否已经被删除
        lambdaQueryWrapper.in(PoTether::getTetherId,tetherIds);
        lambdaQueryWrapper.eq(PoTether::getDelFlag ,1);
        long count2 = this.count(lambdaQueryWrapper);
        if (count2 > 0){
            throw new Exception("套系已经被删除");
        }
        //判断套系是否上链
        lambdaQueryWrapper.in(PoTether::getTetherId,tetherIds);
        lambdaQueryWrapper.eq(PoTether::getCochain,0);
        long count = this.count(lambdaQueryWrapper);
        if (count>0){
                throw new Exception("套系已经上链");
        }
        //判断套系是否上架
        lambdaQueryWrapper.in(PoTether::getTetherId,tetherIds);
        lambdaQueryWrapper.eq(PoTether::getGrounding,0);
        long count1 = this.count(lambdaQueryWrapper);
        if (count1>0){

            throw new Exception("套系已经上架");
        }
        //如果可以删除 先删除套系po_tether表中的数据
        this.poTetherMapper.deletePoTetherByTetherIds(tetherIds);
        //删除po_collection表中的数据
        lambdaQueryWrapper1.in(PoCollection::getTetherId,tetherIds);
        poCollectionService.remove(lambdaQueryWrapper1);
    }
    /**
     * 校验套系名称是否存在
     * @param tetherVo
     * @return
     */

    @Override
    public String checkPostTetherNameUnique(TetherVo tetherVo) {
        Long tetherId = StringUtils.isNull(tetherVo.getTetherId()) ? -1L : tetherVo.getTetherId();
        PoTether info = poTetherMapper.checkPostTetherTitleUnique(tetherVo.getTetherName());
        if (StringUtils.isNotNull(info) && info.getTetherId() != tetherId) {
            return UserConstants.NOT_UNIQUE;
        }
            return UserConstants.UNIQUE;
    }
    /**
     * 检验图片是否重复
     * @param tetherVo
     * @return
     */
    @Override
    public String checkPostTetherImageUnique(TetherVo tetherVo) {

        Long tetherId = StringUtils.isNull(tetherVo.getTetherId()) ? -1L : tetherVo.getTetherId();

        PoTether info = poTetherMapper.checkPostTetherImageUnique(tetherVo.getImage());
        if (StringUtils.isNotNull(info) && info.getTetherId() != tetherId) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;

    }

    /**
     * 批量查询套系信息
     * @param tetherIds
     * @return
     */
    @Override
    public TetherVo selectPoTetherByTetherIdsWithCollection(Long[] tetherIds) {

        //查询套系基本信息 po_tether
        PoTether poTether = this.poTetherMapper.selectPoTetherByTetherIds(tetherIds);
        TetherVo tetherVo = new TetherVo();
        if (poTether != null) {
            BeanUtils.copyProperties(poTether, tetherVo);
        }
        return  tetherVo;
    }

    /**
     * 修改状态 是否上链
     * @param cochain
     * @param tetherId
     * @return
     */
    @Override
    public boolean updatePoTetherIsCochain(Integer cochain, Long tetherId) {

       return  poTetherMapper.updatePoTetherIsCochain(cochain,tetherId);

    }

    /**
     * 修改状态 是否上架
     * @param grounding
     * @param tetherId
     * @return
     */
    @Override
    public boolean updatePoTetherIsGrounding(Integer grounding, Long tetherId) {
        return poTetherMapper.updatePoTetherIsGrounding(grounding, tetherId);
    }

    /**
     * 藏品种类个数
     * @param poTether
     * @return
     */
    @Override
    public List<PoTether> selectPoTetherCollectionList(PoTether poTether) {

        poTetherMapper.updatePostTetherCollection(poTether);
        return poTetherMapper.selectPoTetherCollectionList(poTether);
    }

    /**
     * 已经上架只允许修改展示结束时间
     * @param tetherVo
     */
    @Override
    public void updateShowTimeEnd(TetherVo tetherVo) {
        poTetherMapper.updateShowTimeEnd(tetherVo);
    }

    @Override
    public void updateTime(PoTether poTether) {
         poTetherMapper.updateTime(poTether);
    }

}

